{% extends "base.html" %}

{% block main %}

  <h1>{{ _("Error") }}</h1>

  <aside class="sidebar" id="left">
  </aside>
  <section id="main">
    <article>
      <p>{{ _("Page not found!") }}</p>
    </article>
  </section>
{% endblock main %}
